@@ -115,8 +115,8 @@ class Trainer(BaseTrainer):
         try:
             checkpoint = torch.load(filename, lambda storage, loc: storage)
         except BaseException:
-            logger.exception("Cannot load model from {}".format(filename))
-            sys.exit(1)
+            logger.error("Cannot load model from {}".format(filename))
+            raise
         self.args = checkpoint['config']
         if args: self.args.update(args)
         self.vocab = MultiVocab.load_state_dict(checkpoint['vocab'])
