@@ -4,6 +4,7 @@ Client for accessing Stanford CoreNLP in Python
 
 import atexit
 import contextlib
+import enum
 import io
 import os
 import re
@@ -17,6 +18,9 @@ import time
 import sys
 import uuid
 
+from datetime import datetime
+from pathlib import Path
+
 from six.moves.urllib.parse import urlparse
 
 from stanza.protobuf import Document, parseFromDelimitedString, writeToDelimitedString, to_text
@@ -27,7 +31,9 @@ logger = logging.getLogger('stanza')
 # pattern tmp props file should follow
 SERVER_PROPS_TMP_FILE_PATTERN = re.compile('corenlp_server-(.*).props')
 
-# info for Stanford CoreNLP supported languages
+# Check if str is CoreNLP supported language
+CORENLP_LANGS = ['ar', 'arabic', 'chinese', 'zh', 'english', 'en', 'french', 'fr', 'de', 'german', 'es', 'spanish']
+
 LANGUAGE_SHORTHANDS_TO_FULL = {
     "ar": "arabic",
     "zh": "chinese",
@@ -37,36 +43,24 @@ LANGUAGE_SHORTHANDS_TO_FULL = {
     "es": "spanish"
 }
 
-LANGUAGE_DEFAULT_ANNOTATORS = {
-    "arabic": "tokenize,ssplit,pos,parse",
-    "chinese": "tokenize,ssplit,pos,lemma,ner,parse,coref",
-    "english": "tokenize,ssplit,pos,lemma,ner,depparse",
-    "french": "tokenize,ssplit,pos,depparse",
-    "german": "tokenize,ssplit,pos,ner,parse",
-    "spanish": "tokenize,ssplit,pos,lemma,ner,depparse,kbp"
-}
 
-ENGLISH_DEFAULT_REQUEST_PROPERTIES = {
-    "annotators": "tokenize,ssplit,pos,lemma,ner,depparse",
-    "tokenize.language": "en",
-    "pos.model": "edu/stanford/nlp/models/pos-tagger/english-left3words-distsim.tagger",
-    "ner.model": "edu/stanford/nlp/models/ner/english.all.3class.distsim.crf.ser.gz,"
-                 "edu/stanford/nlp/models/ner/english.muc.7class.distsim.crf.ser.gz,"
-                 "edu/stanford/nlp/models/ner/english.conll.4class.distsim.crf.ser.gz",
-    "sutime.language": "english",
-    "sutime.rules": "edu/stanford/nlp/models/sutime/defs.sutime.txt,"
-                    "edu/stanford/nlp/models/sutime/english.sutime.txt,"
-                    "edu/stanford/nlp/models/sutime/english.holidays.sutime.txt",
-    "ner.applyNumericClassifiers": "true",
-    "ner.useSUTime": "true",
+def is_corenlp_lang(props_str):
+    """ Check if a string references a CoreNLP language """
+    return props_str.lower() in CORENLP_LANGS
 
-    "ner.fine.regexner.mapping": "ignorecase=true,validpospattern=^(NN|JJ).*,"
-                                 "edu/stanford/nlp/models/kbp/english/gazetteers/regexner_caseless.tab;",
-                                 "edu/stanford/nlp/models/kbp/english/gazetteers/regexner_cased.tab"
-    "ner.fine.regexner.noDefaultOverwriteLabels": "CITY",
-    "ner.language": "en",
-    "depparse.model": "edu/stanford/nlp/models/parser/nndep/english_UD.gz"
-}
+
+# Validate CoreNLP properties
+CORENLP_OUTPUT_VALS = ["conll", "conllu", "json", "serialized", "text", "xml", "inlinexml"]
+
+
+def validate_corenlp_props(properties=None, annotators=None, output_format=None):
+    """ Do basic checks to validate CoreNLP properties """
+    if output_format and output_format.lower() not in CORENLP_OUTPUT_VALS:
+        raise ValueError(f"{output_format} not a valid CoreNLP outputFormat value! Choose from: {CORENLP_OUTPUT_VALS}")
+    if type(properties) == dict:
+        if "outputFormat" in properties and properties["outputFormat"].lower() not in CORENLP_OUTPUT_VALS:
+            raise ValueError(f"{properties['outputFormat']} not a valid CoreNLP outputFormat value! Choose from: "
+                             f"{CORENLP_OUTPUT_VALS}")
 
 
 class AnnotationException(Exception):
@@ -88,20 +82,25 @@ class PermanentlyFailedException(Exception):
     """ Exception raised if the service should NOT retry the request. """
     pass
 
+class StartServer(enum.Enum):
+    DONT_START = 0
+    FORCE_START = 1
+    TRY_START = 2
+
 
 def clean_props_file(props_file):
     # check if there is a temp server props file to remove and remove it
     if props_file:
-        if (os.path.isfile(props_file) and
-            SERVER_PROPS_TMP_FILE_PATTERN.match(os.path.basename(props_file))):
+        if os.path.isfile(props_file) and SERVER_PROPS_TMP_FILE_PATTERN.match(os.path.basename(props_file)):
             os.remove(props_file)
 
+
 class RobustService(object):
     """ Service that resuscitates itself if it is not available. """
     CHECK_ALIVE_TIMEOUT = 120
 
     def __init__(self, start_cmd, stop_cmd, endpoint, stdout=sys.stdout,
-                 stderr=sys.stderr, be_quiet=False, host=None, port=None):
+                 stderr=sys.stderr, be_quiet=False, host=None, port=None, ignore_binding_error=False):
         self.start_cmd = start_cmd and shlex.split(start_cmd)
         self.stop_cmd = stop_cmd and shlex.split(stop_cmd)
         self.endpoint = endpoint
@@ -113,11 +112,12 @@ class RobustService(object):
         self.be_quiet = be_quiet
         self.host = host
         self.port = port
+        self.ignore_binding_error = ignore_binding_error
         atexit.register(self.atexit_kill)
 
     def is_alive(self):
         try:
-            if self.server is not None and self.server.poll() is not None:
+            if not self.ignore_binding_error and self.server is not None and self.server.poll() is not None:
                 return False
             return requests.get(self.endpoint + "/ping").ok
         except requests.exceptions.ConnectionError as e:
@@ -130,13 +130,19 @@ class RobustService(object):
                     try:
                         sock.bind((self.host, self.port))
                     except socket.error:
-                        raise PermanentlyFailedException("Error: unable to start the CoreNLP server on port %d (possibly something is already running there)" % self.port)
+                        if self.ignore_binding_error:
+                            logger.info(f"Connecting to existing CoreNLP server at {self.host}:{self.port}")
+                            self.server = None
+                            return
+                        else:
+                            raise PermanentlyFailedException("Error: unable to start the CoreNLP server on port %d "
+                                                         "(possibly something is already running there)" % self.port)
             if self.be_quiet:
                 # Issue #26: subprocess.DEVNULL isn't supported in python 2.7.
                 stderr = open(os.devnull, 'w')
             else:
                 stderr = self.stderr
-            print(f"Starting server with command: {' '.join(self.start_cmd)}")
+            logger.info(f"Starting server with command: {' '.join(self.start_cmd)}")
             self.server = subprocess.Popen(self.start_cmd,
                                            stderr=stderr,
                                            stdout=stderr)
@@ -206,60 +212,116 @@ class RobustService(object):
         self.is_active = True
 
 
+def resolve_classpath(classpath=None):
+    """
+    Returns the classpath to use for corenlp.
+
+    Prefers to use the given classpath parameter, if available.  If
+    not, uses the CORENLP_HOME environment variable.  Resolves $CLASSPATH
+    (the exact string) in either the classpath parameter or $CORENLP_HOME.
+    """
+    if classpath == '$CLASSPATH' or (classpath is None and os.getenv("CORENLP_HOME", None) == '$CLASSPATH'):
+        classpath = os.getenv("CLASSPATH")
+    elif classpath is None:
+        classpath = os.getenv("CORENLP_HOME", os.path.join(str(Path.home()), 'stanza_corenlp'))
+
+        assert os.path.exists(classpath), \
+            "Please install CoreNLP by running `stanza.install_corenlp()`. If you have installed it, please define " \
+            "$CORENLP_HOME to be location of your CoreNLP distribution or pass in a classpath parameter."
+        classpath = os.path.join(classpath, "*")
+    return classpath
+
+
 class CoreNLPClient(RobustService):
-    """ A CoreNLP client to the Stanford CoreNLP server. """
+    """ A client to the Stanford CoreNLP server. """
 
     DEFAULT_ENDPOINT = "http://localhost:9000"
     DEFAULT_TIMEOUT = 60000
     DEFAULT_THREADS = 5
-    DEFAULT_ANNOTATORS = "tokenize,ssplit,pos,lemma,ner,depparse"
     DEFAULT_OUTPUT_FORMAT = "serialized"
     DEFAULT_MEMORY = "5G"
     DEFAULT_MAX_CHAR_LENGTH = 100000
-    DEFAULT_SERIALIZER = "edu.stanford.nlp.pipeline.ProtobufAnnotationSerializer"
-    DEFAULT_INPUT_FORMAT = "text"
-    PIPELINE_LANGUAGES = \
-        ['ar', 'arabic', 'chinese', 'zh', 'english', 'en', 'french', 'fr', 'de', 'german', 'es', 'spanish']
 
-    def __init__(self, start_server=True,
+    def __init__(self, start_server=StartServer.FORCE_START,
                  endpoint=DEFAULT_ENDPOINT,
                  timeout=DEFAULT_TIMEOUT,
                  threads=DEFAULT_THREADS,
                  annotators=None,
-                 properties=None,
                  output_format=None,
+                 properties=None,
                  stdout=sys.stdout,
                  stderr=sys.stderr,
                  memory=DEFAULT_MEMORY,
-                 be_quiet=True,
+                 be_quiet=False,
                  max_char_length=DEFAULT_MAX_CHAR_LENGTH,
                  preload=True,
                  classpath=None,
                  **kwargs):
 
-        # properties cache maps keys to properties dictionaries for convenience
-        self.properties_cache = {}
-        self.server_props_file = {'is_temp': False, 'path': None}
+        # whether or not server should be started by client
+        self.start_server = start_server
+        self.server_props_path = None
+        self.server_start_time = None
+        self.server_host = None
+        self.server_port = None
+        self.server_classpath = None
+        # validate properties
+        validate_corenlp_props(properties=properties, annotators=annotators, output_format=output_format)
+        # set up client defaults
+        self.properties = properties
+        self.annotators = annotators
+        self.output_format = output_format
+        self._setup_client_defaults()
         # start the server
-        if start_server:
+        if isinstance(start_server, bool):
+            warning_msg = f"Setting 'start_server' to a boolean value when constructing {self.__class__.__name__} is deprecated and will stop" + \
+                " to function in a future version of stanza. Please consider switching to using a value from stanza.server.StartServer."
+            logger.warning(warning_msg)
+            start_server = StartServer.FORCE_START if start_server is True else StartServer.DONT_START
+
+        # start the server
+        if start_server is StartServer.FORCE_START or start_server is StartServer.TRY_START:
+            # record info for server start
+            self.server_start_time = datetime.now()
             # set up default properties for server
-            self._setup_default_server_props(properties, annotators, output_format)
-            # at this point self.server_start_info and self.server_props_file should be set
+            self._setup_server_defaults()
             host, port = urlparse(endpoint).netloc.split(":")
             port = int(port)
             assert host == "localhost", "If starting a server, endpoint must be localhost"
-            if classpath == '$CLASSPATH':
-                classpath = os.getenv("CLASSPATH")
-            elif classpath is None:
-                classpath = os.getenv("CORENLP_HOME")
-                assert classpath is not None, \
-                    "Please define $CORENLP_HOME to be location of your CoreNLP distribution or pass in a classpath parameter"
-                classpath = classpath + "/*"
+            classpath = resolve_classpath(classpath)
             start_cmd = f"java -Xmx{memory} -cp '{classpath}'  edu.stanford.nlp.pipeline.StanfordCoreNLPServer " \
                         f"-port {port} -timeout {timeout} -threads {threads} -maxCharLength {max_char_length} " \
-                        f"-quiet {be_quiet} -serverProperties {self.server_props_file['path']}"
-            if preload and self.server_start_info.get('preload_annotators'):
-                start_cmd += f" -preload {self.server_start_info['preload_annotators']}"
+                        f"-quiet {be_quiet} "
+
+            self.server_classpath = classpath
+            self.server_host = host
+            self.server_port = port
+
+            # set up server defaults
+            if self.server_props_path is not None:
+                start_cmd += f" -serverProperties {self.server_props_path}"
+
+            # set annotators for server default
+            if self.annotators is not None:
+                annotators_str = self.annotators if type(annotators) == str else ",".join(annotators)
+                start_cmd += f" -annotators {annotators_str}"
+
+            # specify what to preload, if anything
+            if preload:
+                if type(preload) == bool:
+                    # -preload flag means to preload all default annotators
+                    start_cmd += " -preload"
+                elif type(preload) == list:
+                    # turn list into comma separated list string, only preload these annotators
+                    start_cmd += f" -preload {','.join(preload)}"
+                elif type(preload) == str:
+                    # comma separated list of annotators
+                    start_cmd += f" -preload {preload}"
+
+            # set outputFormat for server default
+            # if no output format requested by user, set to serialized
+            start_cmd += f" -outputFormat {self.output_format}"
+
             # additional options for server:
             # - server_id
             # - ssl
@@ -269,106 +331,92 @@ class CoreNLPClient(RobustService):
             # - key
             # - username
             # - password
-            # - blacklist
+            # - blockList
             for kw in ['ssl', 'strict']:
                 if kwargs.get(kw) is not None:
                     start_cmd += f" -{kw}"
-            for kw in ['status_port', 'uriContext', 'key', 'username', 'password', 'blacklist', 'server_id']:
+            for kw in ['status_port', 'uriContext', 'key', 'username', 'password', 'blockList', 'server_id']:
                 if kwargs.get(kw) is not None:
                     start_cmd += f" -{kw} {kwargs.get(kw)}"
             stop_cmd = None
         else:
             start_cmd = stop_cmd = None
             host = port = None
-            self.server_start_info = {}
 
         super(CoreNLPClient, self).__init__(start_cmd, stop_cmd, endpoint,
-                                            stdout, stderr, be_quiet, host=host, port=port)
+                                            stdout, stderr, be_quiet, host=host, port=port, ignore_binding_error=(start_server == StartServer.TRY_START))
 
         self.timeout = timeout
 
-    def _setup_default_server_props(self, properties, annotators, output_format):
+    def _setup_client_defaults(self):
         """
-        Set up the default properties for the server from either:
+        Do some processing of annotators and output_format specified for the client.
+        If interacting with an externally started server, these will be defaults for annotate() calls.
+        :return: None
+        """
+        # normalize annotators to str
+        if self.annotators is not None:
+            self.annotators = self.annotators if type(self.annotators) == str else ",".join(self.annotators)
+
+        # handle case where no output format is specified
+        if self.output_format is None:
+            if type(self.properties) == dict and 'outputFormat' in self.properties:
+                self.output_format = self.properties['outputFormat']
+            else:
+                self.output_format = CoreNLPClient.DEFAULT_OUTPUT_FORMAT
+
+    def _setup_server_defaults(self):
+        """
+        Set up the default properties for the server.
+
+        The properties argument can take on one of 3 value types
 
         1. File path on system or in CLASSPATH (e.g. /path/to/server.props or StanfordCoreNLP-french.properties
-        2. Stanford CoreNLP supported language (e.g. french)
+        2. Name of a Stanford CoreNLP supported language (e.g. french or fr)
         3. Python dictionary (properties written to tmp file for Java server, erased at end)
-        4. Default (just use standard defaults set server side in Java code, with the exception that the default
-                    default outputFormat is changed to serialized)
 
-        If defaults are being set client side, values of annotators and output_format will overwrite the
-        client side properties.  If the defaults are being set server side, those parameters will be ignored.
+        In addition, an annotators list and output_format can be specified directly with arguments. These
+        will overwrite any settings in the specified properties.
 
-        Info about the properties used to start the server is stored in self.server_start_info
-        If a file is used, info about the file (path, whether tmp or not) is stored in self.server_props_file
+        If no properties are specified, the standard Stanford CoreNLP English server will be launched. The outputFormat
+        will be set to 'serialized' and use the ProtobufAnnotationSerializer.
         """
-        # store information about server start up
-        self.server_start_info = {}
-        # ensure properties is str or dict
-        if properties is None or (not isinstance(properties, str) and not isinstance(properties, dict)):
-            if properties is not None:
-                print('Warning: properties passed invalid value (not a str or dict), setting properties = {}')
-            properties = {}
-        # check if properties is a string
-        if isinstance(properties, str):
-            # translate Stanford CoreNLP language name to properties file if properties is a language name
-            if properties.lower() in CoreNLPClient.PIPELINE_LANGUAGES:
-                lang_name = properties.lower()
-                if lang_name in LANGUAGE_SHORTHANDS_TO_FULL:
-                    lang_name = LANGUAGE_SHORTHANDS_TO_FULL[lang_name]
-                if lang_name in ['en', 'english']:
-                    self.server_props_file['path'] = f'StanfordCoreNLP.properties'
-                else:
-                    self.server_props_file['path'] = f'StanfordCoreNLP-{lang_name}.properties'
-                self.server_start_info['preload_annotators'] = LANGUAGE_DEFAULT_ANNOTATORS[lang_name]
-                print(f"Using Stanford CoreNLP default properties for: {lang_name}.  Make sure to have {lang_name} "
-                      f"models jar (available for download here: https://stanfordnlp.github.io/CoreNLP/) in CLASSPATH")
-            # otherwise assume properties string is a path
-            else:
-                self.server_props_file['path'] = properties
-                if os.path.isfile(properties):
-                    props_from_file = read_corenlp_props(properties)
-                    self.server_start_info['props'] = props_from_file
-                    self.server_start_info['preload_annotators'] = props_from_file.get('annotators')
-                else:
-                    print(f"Warning: {properties} does not correspond to a file path.")
-            print(f"Setting server defaults from: {self.server_props_file['path']}")
-            self.server_start_info['props_file'] = self.server_props_file['path']
-            self.server_start_info['server_side'] = True
-            if annotators is not None:
-                print(f"Warning: Server defaults being set server side, ignoring annotators={annotators}")
-            if output_format is not None:
-                print(f"Warning: Server defaults being set server side, ignoring output_format={output_format}")
-        # check if client side should set default properties
-        else:
-            # set up properties from client side
-            # the Java Stanford CoreNLP server defaults to "json" for outputFormat
-            # but by default servers started by Python interface will override this to return serialized object
-            client_side_properties = {
-                'annotators': CoreNLPClient.DEFAULT_ANNOTATORS,
-                'outputFormat': CoreNLPClient.DEFAULT_OUTPUT_FORMAT,
-                'serializer': CoreNLPClient.DEFAULT_SERIALIZER
-            }
-            client_side_properties.update(properties)
-            # override if a specific annotators list was specified
-            if annotators:
-                client_side_properties['annotators'] = \
-                    ",".join(annotators) if isinstance(annotators, list) else annotators
-            # override if a specific output format was specified
-            if output_format is not None and isinstance(output_format, str):
-                client_side_properties['outputFormat'] = output_format
-            # write client side props to a tmp file which will be erased at end
-            self.server_props_file['path'] = write_corenlp_props(client_side_properties)
-            atexit.register(clean_props_file, self.server_props_file['path'])
-            self.server_props_file['is_temp'] = True
-            # record server start up info
-            self.server_start_info['client_side'] = True
-            self.server_start_info['props'] = client_side_properties
-            self.server_start_info['props_file'] = self.server_props_file['path']
-            self.server_start_info['preload_annotators'] = client_side_properties['annotators']
 
-    def _request(self, buf, properties, **kwargs):
+        # ensure properties is str or dict
+        if self.properties is None or (not isinstance(self.properties, str) and not isinstance(self.properties, dict)):
+            if self.properties is not None:
+                logger.warning('properties passed invalid value (not a str or dict), setting properties = {}')
+            self.properties = {}
+        # check if properties is a string, pass on to server which can handle
+        if isinstance(self.properties, str):
+            # try to translate to Stanford CoreNLP language name, or assume properties is a path
+            if is_corenlp_lang(self.properties):
+                if self.properties.lower() in LANGUAGE_SHORTHANDS_TO_FULL:
+                    self.properties = LANGUAGE_SHORTHANDS_TO_FULL[self.properties]
+                logger.info(
+                    f"Using Stanford CoreNLP default properties for: {self.properties}.  Make sure to have "
+                    f"{self.properties} models jar (available for download here: "
+                    f"https://stanfordnlp.github.io/CoreNLP/) in CLASSPATH")
+            else:
+                if not os.path.isfile(self.properties):
+                    logger.warning(f"{self.properties} does not correspond to a file path. Make sure this file is in "
+                                   f"your CLASSPATH.")
+            self.server_props_path = self.properties
+        elif isinstance(self.properties, dict):
+            # make a copy
+            server_start_properties = dict(self.properties)
+            if self.annotators is not None:
+                server_start_properties['annotators'] = self.annotators
+            if self.output_format is not None and isinstance(self.output_format, str):
+                server_start_properties['outputFormat'] = self.output_format
+            # write desired server start properties to tmp file
+            # set up to erase on exit
+            tmp_path = write_corenlp_props(server_start_properties)
+            logger.info(f"Writing properties to tmp file: {tmp_path}")
+            atexit.register(clean_props_file, tmp_path)
+            self.server_props_path = tmp_path
+
+    def _request(self, buf, properties, reset_default=False, **kwargs):
         """
         Send a request to the CoreNLP server.
 
@@ -392,7 +440,7 @@ class CoreNLPClient(RobustService):
                 kwargs.pop('username')
                 kwargs.pop('password')
             r = requests.post(self.endpoint,
-                              params={'properties': str(properties)},
+                              params={'properties': str(properties), 'resetDefault': str(reset_default).lower()},
                               data=buf, headers={'content-type': ctype},
                               timeout=(self.timeout*2)/1000, **kwargs)
             r.raise_for_status()
@@ -403,71 +451,67 @@ class CoreNLPClient(RobustService):
             else:
                 raise AnnotationException(r.text)
 
-    def register_properties_key(self, key, props):
-        """ Register a properties dictionary with a key in the client's properties_cache """
-        if key in CoreNLPClient.PIPELINE_LANGUAGES:
-            print(f'Key {key} not registered in properties cache.  Names of Stanford CoreNLP supported languages are '
-                  f'reserved for Stanford CoreNLP defaults for that language.  For instance "french" or "fr" '
-                  f'corresponds to using the defaults in StanfordCoreNLP-french.properties which are stored with the '
-                  f'server.  If you want to store custom defaults for that language, it is suggested to use a key like '
-                  f' "fr-custom", etc...'
-                  )
-        else:
-            self.properties_cache[key] = props
-
-    def annotate(self, text, annotators=None, output_format=None, properties_key=None, properties=None, **kwargs):
+    def annotate(self, text, annotators=None, output_format=None, properties=None, reset_default=None, **kwargs):
         """
         Send a request to the CoreNLP server.
 
         :param (str | unicode) text: raw text for the CoreNLPServer to parse
         :param (list | string) annotators: list of annotators to use
         :param (str) output_format: output type from server: serialized, json, text, conll, conllu, or xml
-        :param (str) properties_key: key into properties cache for the client
         :param (dict) properties: additional request properties (written on top of defaults)
+        :param (bool) reset_default: don't use server defaults
 
-        The properties for a request are written in this order:
+        Precedence for settings:
 
-        1. Server default properties (server side)
-        2. Properties from client's properties_cache corresponding to properties_key (client side)
-           If the properties_key is the name of a Stanford CoreNLP supported language:
-           [Arabic, Chinese, English, French, German, Spanish], the Stanford CoreNLP defaults will be used (server side)
-        3. Additional properties corresponding to properties (client side)
-        4. Special case specific properties: annotators, output_format (client side)
+        1. annotators and output_format args
+        2. Values from properties dict
+        3. Client defaults self.annotators and self.output_format (set during client construction)
+        4. Server defaults
+
+        Additional request parameters (apart from CoreNLP pipeline properties) such as 'username' and 'password'
+        can be specified with the kwargs.
 
         :return: request result
         """
-        # set properties for server call
-        # first look for a cached default properties set
-        # if a Stanford CoreNLP supported language is specified, just pass {pipelineLanguage="french"}
-        if properties_key is not None:
-            if properties_key.lower() in ['en', 'english']:
-                request_properties = dict(ENGLISH_DEFAULT_REQUEST_PROPERTIES)
-            elif properties_key.lower() in CoreNLPClient.PIPELINE_LANGUAGES:
-                request_properties = {'pipelineLanguage': properties_key.lower()}
-            elif properties_key not in self.properties_cache:
-                raise ValueError("Properties cache does not have '%s'" % properties_key)
+
+        # validate request properties
+        validate_corenlp_props(properties=properties, annotators=annotators, output_format=output_format)
+        # set request properties
+        request_properties = {}
+
+        # start with client defaults
+        if self.annotators is not None:
+            request_properties['annotators'] = self.annotators
+        if self.output_format is not None:
+            request_properties['outputFormat'] = self.output_format
+
+        # add values from properties arg
+        # handle str case
+        if type(properties) == str:
+            if is_corenlp_lang(properties):
+                properties = {'pipelineLanguage': properties.lower()}
+                if reset_default is None:
+                    reset_default = True
             else:
-                request_properties = dict(self.properties_cache[properties_key])
-        else:
-            request_properties = {}
-        # add on custom properties for this request
-        if properties is None:
-            properties = {}
-        request_properties.update(properties)
+                raise ValueError(f"Unrecognized properties keyword {properties}")
+
+        if type(properties) == dict:
+            request_properties.update(properties)
+
         # if annotators list is specified, override with that
-        if annotators is not None:
-            request_properties['annotators'] = ",".join(annotators) if isinstance(annotators, list) else annotators
-        # always send an output format with request
-        # in some scenario's the server's default output format is unknown, so default to serialized
-        if output_format is not None:
+        # also can use the annotators field the object was created with
+        if annotators is not None and (type(annotators) == str or type(annotators) == list):
+            request_properties['annotators'] = annotators if type(annotators) == str else ",".join(annotators)
+
+        # if output format is specified, override with that
+        if output_format is not None and type(output_format) == str:
             request_properties['outputFormat'] = output_format
-        if request_properties.get('outputFormat') is None:
-            if self.server_start_info.get('props', {}).get('outputFormat'):
-                request_properties['outputFormat'] = self.server_start_info['props']['outputFormat']
-            else:
-                request_properties['outputFormat'] = CoreNLPClient.DEFAULT_OUTPUT_FORMAT
+
         # make the request
-        r = self._request(text.encode('utf-8'), request_properties, **kwargs)
+        # if not explictly set or the case of pipelineLanguage, reset_default should be None
+        if reset_default is None:
+            reset_default = False
+        r = self._request(text.encode('utf-8'), request_properties, reset_default, **kwargs)
         if request_properties["outputFormat"] == "json":
             return r.json()
         elif request_properties["outputFormat"] == "serialized":
@@ -488,7 +532,7 @@ class CoreNLPClient(RobustService):
                 'serializer': 'edu.stanford.nlp.pipeline.ProtobufAnnotationSerializer'
             })
         if annotators:
-            properties['annotators'] = ",".join(annotators) if isinstance(annotators, list) else annotators
+            properties['annotators'] = annotators if type(annotators) == str else ",".join(annotators)
         with io.BytesIO() as stream:
             writeToDelimitedString(doc, stream)
             msg = stream.getvalue()
