@@ -34,32 +34,16 @@ Sentence #1 (10 tokens):
 Angela Merkel ist seit 2005 Bundeskanzlerin der Bundesrepublik Deutschland.
 
 Tokens:
-[Text=Angela CharacterOffsetBegin=0 CharacterOffsetEnd=6 PartOfSpeech=PROPN Lemma=angela NamedEntityTag=PERSON]
-[Text=Merkel CharacterOffsetBegin=7 CharacterOffsetEnd=13 PartOfSpeech=PROPN Lemma=merkel NamedEntityTag=PERSON]
-[Text=ist CharacterOffsetBegin=14 CharacterOffsetEnd=17 PartOfSpeech=AUX Lemma=ist NamedEntityTag=O]
-[Text=seit CharacterOffsetBegin=18 CharacterOffsetEnd=22 PartOfSpeech=ADP Lemma=seit NamedEntityTag=O]
-[Text=2005 CharacterOffsetBegin=23 CharacterOffsetEnd=27 PartOfSpeech=NUM Lemma=2005 NamedEntityTag=O]
-[Text=Bundeskanzlerin CharacterOffsetBegin=28 CharacterOffsetEnd=43 PartOfSpeech=NOUN Lemma=bundeskanzlerin NamedEntityTag=O]
-[Text=der CharacterOffsetBegin=44 CharacterOffsetEnd=47 PartOfSpeech=DET Lemma=der NamedEntityTag=O]
-[Text=Bundesrepublik CharacterOffsetBegin=48 CharacterOffsetEnd=62 PartOfSpeech=PROPN Lemma=bundesrepublik NamedEntityTag=LOCATION]
-[Text=Deutschland CharacterOffsetBegin=63 CharacterOffsetEnd=74 PartOfSpeech=PROPN Lemma=deutschland NamedEntityTag=LOCATION]
-[Text=. CharacterOffsetBegin=74 CharacterOffsetEnd=75 PartOfSpeech=PUNCT Lemma=. NamedEntityTag=O]
-
-Dependency Parse (enhanced plus plus dependencies):
-root(ROOT-0, Bundeskanzlerin-6)
-nsubj(Bundeskanzlerin-6, Angela-1)
-flat(Angela-1, Merkel-2)
-cop(Bundeskanzlerin-6, ist-3)
-case(2005-5, seit-4)
-nmod:seit(Bundeskanzlerin-6, 2005-5)
-det(Bundesrepublik-8, der-7)
-nmod(Bundeskanzlerin-6, Bundesrepublik-8)
-appos(Bundesrepublik-8, Deutschland-9)
-punct(Bundeskanzlerin-6, .-10)
-
-Extracted the following NER entity mentions:
-Angela Merkel   PERSON  PERSON:0.9999981583355767
-Bundesrepublik Deutschland      LOCATION        LOCATION:0.968290232887181
+[Text=Angela CharacterOffsetBegin=0 CharacterOffsetEnd=6 PartOfSpeech=PROPN]
+[Text=Merkel CharacterOffsetBegin=7 CharacterOffsetEnd=13 PartOfSpeech=PROPN]
+[Text=ist CharacterOffsetBegin=14 CharacterOffsetEnd=17 PartOfSpeech=AUX]
+[Text=seit CharacterOffsetBegin=18 CharacterOffsetEnd=22 PartOfSpeech=ADP]
+[Text=2005 CharacterOffsetBegin=23 CharacterOffsetEnd=27 PartOfSpeech=NUM]
+[Text=Bundeskanzlerin CharacterOffsetBegin=28 CharacterOffsetEnd=43 PartOfSpeech=NOUN]
+[Text=der CharacterOffsetBegin=44 CharacterOffsetEnd=47 PartOfSpeech=DET]
+[Text=Bundesrepublik CharacterOffsetBegin=48 CharacterOffsetEnd=62 PartOfSpeech=PROPN]
+[Text=Deutschland CharacterOffsetBegin=63 CharacterOffsetEnd=74 PartOfSpeech=PROPN]
+[Text=. CharacterOffsetBegin=74 CharacterOffsetEnd=75 PartOfSpeech=PUNCT]
 """
 
 FRENCH_CUSTOM_PROPS = {'annotators': 'tokenize,ssplit,mwt,pos,parse',
@@ -120,28 +104,6 @@ Constituency parse:
     (NP (DET quelques) (NOUN jours))
     (AdP (ADV plus) (ADV tôt))
     (PUNCT .)))
-
-
-Binary Constituency parse: 
-(ROOT
-  (SENT
-    (NP (DET Cette)
-      (MWN (NOUN enquête) (ADJ préliminaire)))
-    (@SENT
-      (@SENT
-        (@SENT
-          (@SENT
-            (VN
-              (MWV (VERB fait) (NOUN suite)))
-            (PP (ADP à)
-              (NP
-                (@NP (DET les) (NOUN révélations))
-                (PP (ADP de)
-                  (NP (NOUN l’)
-                    (AP (ADJ hebdomadaire)))))))
-          (NP (DET quelques) (NOUN jours)))
-        (AdP (ADV plus) (ADV tôt)))
-      (PUNCT .))))
 """
 
 FRENCH_EXTRA_GOLD = """
@@ -228,7 +190,6 @@ punct(presidente-7, .-10)
 def corenlp_client():
     """ Client to run tests on """
     client = corenlp.CoreNLPClient(annotators='tokenize,ssplit,pos', server_id='stanza_request_tests_server')
-    client.register_properties_key('fr-custom', FRENCH_CUSTOM_PROPS)
     yield client
     client.stop()
 
@@ -245,33 +206,13 @@ def test_python_dict(corenlp_client):
     """ Test using a Python dictionary to specify all request properties """
     ann = corenlp_client.annotate(ES_DOC, properties=ES_PROPS, output_format="text")
     assert ann.strip() == ES_PROPS_GOLD.strip()
-
-
-def test_properties_key_and_python_dict(corenlp_client):
-    """ Test using a properties key and additional properties """
-    ann = corenlp_client.annotate(FRENCH_DOC, properties_key='fr-custom', properties=FRENCH_EXTRA_PROPS)
-    assert ann.strip() == FRENCH_EXTRA_GOLD.strip()
-
-
-def test_properties_key(corenlp_client):
-    """ Test using the properties_key which was registered with the properties cache """
-    ann = corenlp_client.annotate(FRENCH_DOC, properties_key='fr-custom')
-    assert ann.strip() == FRENCH_CUSTOM_GOLD.strip()
-
-
-def test_switching_back_and_forth(corenlp_client):
-    """ Test using a properties key, then properties key with python dict, then back to just properties key """
-    ann = corenlp_client.annotate(FRENCH_DOC, properties_key='fr-custom')
-    assert ann.strip() == FRENCH_CUSTOM_GOLD.strip()
-    ann = corenlp_client.annotate(FRENCH_DOC, properties_key='fr-custom', properties=FRENCH_EXTRA_PROPS)
-    assert ann.strip() == FRENCH_EXTRA_GOLD.strip()
-    ann = corenlp_client.annotate(FRENCH_DOC, properties_key='fr-custom')
+    ann = corenlp_client.annotate(FRENCH_DOC, properties=FRENCH_CUSTOM_PROPS)
     assert ann.strip() == FRENCH_CUSTOM_GOLD.strip()
 
 
 def test_lang_setting(corenlp_client):
     """ Test using a Stanford CoreNLP supported languages as a properties key """
-    ann = corenlp_client.annotate(GERMAN_DOC, properties_key="german", output_format="text")
+    ann = corenlp_client.annotate(GERMAN_DOC, properties="german", output_format="text")
     compare_ignoring_whitespace(ann, GERMAN_DOC_GOLD)
 
 
